@@ -64,6 +64,12 @@ enum
 enum
 {
   PROP_0,
+#if GTK_CHECK_VERSION (2, 91, 2)
+  PROP_HADJUSTMENT,
+  PROP_VADJUSTMENT,
+  PROP_HSCROLL_POLICY,
+  PROP_VSCROLL_POLICY,
+#endif
   PROP_ACTIVE_CHAR,
   PROP_CODEPOINT_LIST,
   PROP_FONT_DESC,
@@ -138,7 +144,12 @@ G_DEFINE_TYPE (GucharmapChartableAccessibleFactory, gucharmap_chartable_accessib
 
 /* Type definition */
 
+#if GTK_CHECK_VERSION (2, 91, 2)
+G_DEFINE_TYPE_WITH_CODE (GucharmapChartable, gucharmap_chartable, GTK_TYPE_DRAWING_AREA,
+                         G_IMPLEMENT_INTERFACE(GTK_TYPE_SCROLLABLE, NULL))
+#else
 G_DEFINE_TYPE (GucharmapChartable, gucharmap_chartable, GTK_TYPE_DRAWING_AREA)
+#endif
 
 /* utility functions */
 
@@ -1772,6 +1783,36 @@ gucharmap_chartable_size_allocate (GtkWidget *widget,
   update_scrollbar_adjustment (chartable);
 }
 
+#if GTK_CHECK_VERSION (2, 91, 0)
+
+static void
+gucharmap_chartable_get_preferred_width (GtkWidget *widget,
+                                         gint      *minimum,
+                                         gint      *natural)
+{
+  GucharmapChartable *chartable = GUCHARMAP_CHARTABLE (widget);
+  int font_size_px;
+
+  font_size_px = get_font_size_px (chartable);
+
+  *minimum = *natural = FACTOR_WIDTH * font_size_px;
+}
+
+static void
+gucharmap_chartable_get_preferred_height (GtkWidget *widget,
+                                          gint      *minimum,
+                                          gint      *natural)
+{
+  GucharmapChartable *chartable = GUCHARMAP_CHARTABLE (widget);
+  int font_size_px;
+
+  font_size_px = get_font_size_px (chartable);
+
+  *minimum = *natural = FACTOR_HEIGHT * font_size_px;
+}
+
+#else
+
 static void
 gucharmap_chartable_size_request (GtkWidget *widget,
                                   GtkRequisition *requisition)
@@ -1785,6 +1826,8 @@ gucharmap_chartable_size_request (GtkWidget *widget,
   requisition->height = FACTOR_HEIGHT * font_size_px;
 }
 
+#endif /* GTK 3.0 */
+
 static void
 gucharmap_chartable_style_set (GtkWidget *widget, 
                                GtkStyle *previous_style)
@@ -1859,9 +1902,25 @@ gucharmap_chartable_get_accessible (GtkWidget *widget)
 
 /* GucharmapChartable class methods */
 
+#if GTK_CHECK_VERSION (2, 91, 2)
 static void
-gucharmap_chartable_set_adjustments (GucharmapChartable *chartable,
-                                     GtkAdjustment *hadjustment,
+gucharmap_chartable_set_hadjustment (GucharmapChartable *chartable,
+                                     GtkAdjustment *hadjustment)
+{
+  GucharmapChartablePrivate *priv = chartable->priv;
+
+  if (hadjustment == priv->hadjustment)
+    return;
+
+  if (priv->hadjustment)
+    g_object_unref (priv->hadjustment);
+
+  priv->hadjustment = hadjustment ? g_object_ref_sink (hadjustment) : NULL;
+}
+#endif
+
+static void
+gucharmap_chartable_set_vadjustment (GucharmapChartable *chartable,
                                      GtkAdjustment *vadjustment)
 {
   GucharmapChartablePrivate *priv = chartable->priv;
@@ -1892,6 +1951,16 @@ gucharmap_chartable_set_adjustments (GucharmapChartable *chartable,
   update_scrollbar_adjustment (chartable);
 }
 
+#if !GTK_CHECK_VERSION (2, 91, 2)
+static void
+gucharmap_chartable_set_adjustments (GucharmapChartable *chartable,
+                                     GtkAdjustment *hadjustment,
+                                     GtkAdjustment *vadjustment)
+{
+  gucharmap_chartable_set_vadjustment (chartable, vadjustment);
+}
+#endif
+
 static void
 gucharmap_chartable_add_move_binding (GtkBindingSet  *binding_set,
                                       guint           keyval,
@@ -2081,7 +2150,7 @@ gucharmap_chartable_init (GucharmapChartable *chartable)
   GucharmapChartablePrivate *priv;
 
   priv = chartable->priv = G_TYPE_INSTANCE_GET_PRIVATE (chartable, GUCHARMAP_TYPE_CHARTABLE, GucharmapChartablePrivate);
-  
+
   priv->page_first_cell = 0;
   priv->active_cell = 0;
   priv->rows = 1;
@@ -2091,6 +2160,13 @@ gucharmap_chartable_init (GucharmapChartable *chartable)
   priv->snap_pow2_enabled = FALSE;
   priv->font_fallback = TRUE;
 
+  priv->vadjustment = NULL;
+#if GTK_CHECK_VERSION (2, 91, 2)
+  priv->hadjustment = NULL;
+  priv->hscroll_policy = GTK_SCROLL_NATURAL;
+  priv->vscroll_policy = GTK_SCROLL_NATURAL;
+#endif
+
 /* This didn't fix the slow expose events either: */
 /*  gtk_widget_set_double_buffered (widget, FALSE); */
 
@@ -2147,8 +2223,25 @@ gucharmap_chartable_set_property (GObject *object,
                                   GParamSpec *pspec)
 {
   GucharmapChartable *chartable = GUCHARMAP_CHARTABLE (object);
+  GucharmapChartablePrivate *priv = chartable->priv;
 
   switch (prop_id) {
+#if GTK_CHECK_VERSION (2, 91, 2)
+    case PROP_HADJUSTMENT:
+      gucharmap_chartable_set_hadjustment (chartable, g_value_get_object (value));
+      break;
+    case PROP_VADJUSTMENT:
+      gucharmap_chartable_set_vadjustment (chartable, g_value_get_object (value));
+      break;
+    case PROP_HSCROLL_POLICY:
+      priv->hscroll_policy = g_value_get_enum (value);
+      gtk_widget_queue_resize_no_redraw (GTK_WIDGET (chartable));
+      break;
+    case PROP_VSCROLL_POLICY:
+      priv->vscroll_policy = g_value_get_enum (value);
+      gtk_widget_queue_resize_no_redraw (GTK_WIDGET (chartable));
+      break;
+#endif
     case PROP_ACTIVE_CHAR:
       gucharmap_chartable_set_active_character (chartable, g_value_get_uint (value));
       break;
@@ -2186,6 +2279,20 @@ gucharmap_chartable_get_property (GObject *object,
   GucharmapChartablePrivate *priv = chartable->priv;
 
   switch (prop_id) {
+#if GTK_CHECK_VERSION (2, 91, 2)
+    case PROP_HADJUSTMENT:
+      g_value_set_object (value, NULL);
+      break;
+    case PROP_VADJUSTMENT:
+      g_value_set_object (value, priv->vadjustment);
+      break;
+    case PROP_HSCROLL_POLICY:
+      g_value_set_enum (value, priv->hscroll_policy);
+      break;
+    case PROP_VSCROLL_POLICY:
+      g_value_set_enum (value, priv->vscroll_policy);
+      break;
+#endif
     case PROP_ACTIVE_CHAR:
       g_value_set_uint (value, gucharmap_chartable_get_active_character (chartable));
       break;
@@ -2231,9 +2338,12 @@ gucharmap_chartable_class_init (GucharmapChartableClass *klass)
   widget_class->drag_data_received = gucharmap_chartable_drag_data_received;
   widget_class->button_press_event = gucharmap_chartable_button_press;
   widget_class->button_release_event = gucharmap_chartable_button_release;
-#if GTK_CHECK_VERSION (2, 90, 8)
+#if GTK_CHECK_VERSION (2, 91, 0)
+  widget_class->get_preferred_width = gucharmap_chartable_get_preferred_width;
+  widget_class->get_preferred_height = gucharmap_chartable_get_preferred_height;
   widget_class->draw = gucharmap_chartable_draw;
 #else
+  widget_class->size_request = gucharmap_chartable_size_request;
   widget_class->expose_event = gucharmap_chartable_expose_event;
 #endif
   widget_class->focus_in_event = gucharmap_chartable_focus_in_event;
@@ -2242,13 +2352,11 @@ gucharmap_chartable_class_init (GucharmapChartableClass *klass)
   widget_class->key_release_event = gucharmap_chartable_key_release_event;
   widget_class->motion_notify_event = gucharmap_chartable_motion_notify;
   widget_class->size_allocate = gucharmap_chartable_size_allocate;
-  widget_class->size_request = gucharmap_chartable_size_request;
   widget_class->style_set = gucharmap_chartable_style_set;
 #ifdef ENABLE_ACCESSIBLE
   widget_class->get_accessible = gucharmap_chartable_get_accessible;
 #endif
 
-  klass->set_scroll_adjustments = gucharmap_chartable_set_adjustments;
   klass->move_cursor = gucharmap_chartable_move_cursor;
   klass->activate = NULL;
   klass->copy_clipboard = gucharmap_chartable_copy_clipboard;
@@ -2265,6 +2373,17 @@ gucharmap_chartable_class_init (GucharmapChartableClass *klass)
                   G_TYPE_NONE,
                   0);
 
+#if GTK_CHECK_VERSION (2, 91, 2)
+  /* GtkScrollable interface properties */
+  g_object_class_override_property (object_class, PROP_HADJUSTMENT, "hadjustment");
+  g_object_class_override_property (object_class, PROP_VADJUSTMENT, "vadjustment");
+  g_object_class_override_property (object_class, PROP_HSCROLL_POLICY, "hscroll-policy");
+  g_object_class_override_property (object_class, PROP_VSCROLL_POLICY, "vscroll-policy");
+
+#else
+
+  klass->set_scroll_adjustments = gucharmap_chartable_set_adjustments;
+
   /**
    * GucharmapChartable::set-scroll-adjustments
    * @horizontal: the horizontal #GtkAdjustment
@@ -2283,6 +2402,7 @@ gucharmap_chartable_class_init (GucharmapChartableClass *klass)
                   _gucharmap_marshal_VOID__OBJECT_OBJECT,
                   G_TYPE_NONE, 2,
                   GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
+#endif /* GTK 3.0 */
 
   signals[STATUS_MESSAGE] =
     g_signal_new (I_("status-message"), gucharmap_chartable_get_type (), G_SIGNAL_RUN_FIRST,
@@ -2663,7 +2783,7 @@ gucharmap_chartable_get_active_character (GucharmapChartable *chartable)
 }
 
 /**
- * gucharmap_chartable_get_active_character:
+ * gucharmap_chartable_set_active_character:
  * @chartable: a #GucharmapChartable
  * @wc: a unicode character (UTF-32)
  *
@@ -2718,9 +2838,9 @@ gucharmap_chartable_get_snap_pow2 (GucharmapChartable *chartable)
 }
 
 /**
- * gucharmap_chartable_get_active_character:
+ * gucharmap_chartable_set_codepoint_list:
  * @chartable: a #GucharmapChartable
- * @list: a #GucharmapCodepointList
+ * @codepoint_list: a #GucharmapCodepointList
  *
  * Sets the codepoint list to show in the character table.
  */
@@ -2762,7 +2882,7 @@ gucharmap_chartable_set_codepoint_list (GucharmapChartable     *chartable,
  * gucharmap_chartable_get_codepoint_list:
  * @chartable: a #GucharmapChartable
  *
- * Returns: the current codepoint list
+ * Returns: (transfer none): the current codepoint list
  */
 GucharmapCodepointList *
 gucharmap_chartable_get_codepoint_list (GucharmapChartable *chartable)
